#include <iostream>
#include <vector>
#include <cstdlib>
#include <unistd.h>
#include <signal.h>

#include "RtMidi.h"

using namespace std;

bool done;

static void finish(int ignore) { done = 1;}

void midicb( double timeStamp, std::vector<unsigned char> *message, void *userData)
{
    std::vector<unsigned char> msg = *message;
    cout << "Timestamp :" << timeStamp << std::endl;
    try {
        if ((msg[0] & 0xF0) == 0x90) {// Check for note on msgs
            cout << "Note on: " << (unsigned int) (msg[0] & 0x0f) + 1
                    << "   " << (unsigned int) msg[1] << "  "
                    << (unsigned int) msg[2] << endl;
        }
        if ((msg[0] & 0xF0) == 0x80) {// Check for note off msgs
            cout << "Note off: " << (unsigned int) (msg[0] & 0x0f) + 1
                    << "   " <<  (unsigned int) msg[1] << "  "
                    << (unsigned int) msg[2] << endl;
        }
        if ((msg[0] & 0xF0) == 0xb0) {// Check for control msgs
            cout << "Control: " << (unsigned int) (msg[0] & 0x0f) + 1
                    << "   " <<  (unsigned int) msg[1] << "  "
                    << (unsigned int) msg[2] << endl;
        }
        if ((msg[0] & 0xF0) == 0xe0) {// Check for pitch bend
            int pb = ((msg[2] << 7) | msg[1] ) - 8192;
            cout << "Pitch bend: " << (unsigned int) (msg[0] & 0x0f) + 1
                    << "   " << pb << endl;
        }
    } catch (RtMidiError &error) {
        error.printMessage();
        //goto cleanup;
    }
}


int main()
{
    RtMidiIn *midiin = 0;
    int portNumber = 1;

    try {
        midiin = new RtMidiIn();
    } catch (RtMidiError &error) {
        error.printMessage();
        return -1;
    }

    midiin->setCallback(midicb, NULL);

    midiin->openPort(portNumber);

    cout << "Reading MIDI from port : " << midiin->getPortName(portNumber) << endl;

    done = 0;
    (void) signal(SIGINT, finish);

    while(!done) {


        usleep(10000);
    }


cleanup:
    midiin->closePort();
    delete midiin;
    cout << "Done." << endl;

    return 0;
}

